(define-module (ui portsctrl ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :use-module (alterator plist)
               :use-module (srfi srfi-1)
               :export (init))

; Debug string (paste in for your needs).
;(format #t "XXX: ~S\n" rdp_profile_name)

(define *po-domain* "alterator-ports-access")

(define *yesno*
  `(("yes" name "yes" label ,(_ "Yes" *po-domain*))
    ("no" name "no" label ,(_ "No" *po-domain*))))

(define *modes*
  `(("default" name "default" label ,(_ "Default" *po-domain*))
    ("0400" name "0400" label ,(_ "User r/o" *po-domain*))
    ("0440" name "0440" label ,(_ "User + group r/o" *po-domain*))
    ("0600" name "0600" label ,(_ "User r/w" *po-domain*))
    ("0640" name "0640" label ,(_ "User r/w, group r/o" *po-domain*))
    ("0660" name "0660" label ,(_ "User + group r/w" *po-domain*))))

(define *serial-edit-inputs*
  '("list_serial_enabled" "input_serial_owner" "input_serial_group"
    "list_serial_mode"))

(define *serial-edit-buttons*
  '("btn_serial_save"))

(define *usb-edit-inputs*
  '("input_usb_vendor_id" "input_usb_product_id" "input_usb_serial"
	"list_usb_class" "list_usb_subclass" "list_usb_protocol"
    "input_usb_owner" "input_usb_group"
    "list_usb_mode"))

(define *usb-edit-buttons*
  '("btn_usb_save_rule" "btn_usb_rule_remove"))

(define *prsnt-buttons*
  '("btn_prsnt_dev_add"))

(define (onoff-widgets status widgets . more)
  (for-each (lambda (w)
              (if (list? w)
                (onoff-widgets status w)
                (form-update-activity w status)))
            (append widgets more)))

(define (enable-widgets widgets . more)
  (onoff-widgets #t widgets more))

(define (disable-widgets widgets . more)
  (onoff-widgets #f widgets more))

; Select present USB device
(define (usb-prsnt-dev-selected)
  (enable-widgets *prsnt-buttons*))

; Update widgets status
(define (widgets-update)
  (format #t "[ajax,portsctrl] Update widgets status\n")
  (disable-widgets *serial-edit-inputs* *serial-edit-buttons*
                   *usb-edit-inputs* *usb-edit-buttons*
                   *prsnt-buttons*)
  (form-update-value "label_serial_selected" (_ "please choose from the list above" "alterator-ports-access")))


; Reset all values
(define (widgets-reset)
  (format #t "[ajax,portsctrl] Reset all widgets\n")

  (form-update-enum "list_serial_mode" *modes*)
  (form-update-enum "list_usb_mode" *modes*)
  (form-update-enum "list_serial_enabled" *yesno*)

  (for-each (lambda(p) (form-update-value p ""))
            (append '("label_usb_control"
                      "label_serial_control"
                      "label_serial_selected")
                    *serial-edit-inputs*
                    *usb-edit-inputs*)))


; Allow USB hid
(define (allow-usb-hid)

  (catch/message
    (lambda()
      (woo-write "/portsctrl/usb_hid"
                 'allow_hid (form-value "bool_allow_hid")))))

(define *prsnt-devices-list* '())

(define (find-prsnt-device name)
  (find (lambda (x)
		  (equal? name (cdr (plistq 'name (cdr x)))))
		*prsnt-devices-list*))

; Force scan USB device
(define (prsnt-scan force)
  (if (or force (null? *prsnt-devices-list*))
	(begin
	  (format #t
			  "[ajax,portsctrl] Scan present devices (force: ~A)...~%"
			  force)
	  (set! *prsnt-devices-list*
			(woo-list "/portsctrl/list_prsnt_devices"
					  'force (if force "yes" "no")
					  'list_ifaces "yes"
					  'language (form-value "language")))))

  (form-update-enum "list_prsnt_devices" *prsnt-devices-list*)
  (js 'ports_access_process_prsnt)
  (disable-widgets *prsnt-buttons*)
  (form-bind "list_prsnt_devices" "change" usb-prsnt-dev-selected))

; Read current settings
(define (read-current-status)
  (format #t "[ajax,portsctrl] Read current status\n")

  (and

    ; Reset widgets
    (widgets-reset)

    ; Fill lists
    (for-each
      (lambda(lst)
        (form-update-enum lst
                          (woo-list
                            (string-append "/portsctrl/" lst)
                            'language (form-value "language"))))
      '("list_serial" "list_usb_rules"))

	(prsnt-scan #f)

    ; Rebind actions on lists
    (form-bind "list_serial" "change" serial-edit)
    (form-bind "list_usb_rules" "change" usb-edit)

    ; Read current settings
    (let* ((settings
             (woo-read-first "/portsctrl/status"
                             'language (form-value "language")))
           (usb-ctrl-on (woo-get-option settings 'usb_ctrl_on #f))
           (usb-hid (woo-get-option settings 'usb_hid #f))
           (serial-ctrl-on (woo-get-option settings 'serial_ctrl_on #f)))

      (form-update-value "label_serial_control" (if serial-ctrl-on (_ "Serial port control activated" "alterator-ports-access") (_ "Serial port control deactivated" "alterator-ports-access")))
      (form-update-value "label_usb_control" (if usb-ctrl-on (_ "USB port control activated" "alterator-ports-access") (_ "USB port control deactivated" "alterator-ports-access")))
      (form-update-value "bool_allow_hid" usb-hid)
      (form-update-visibility "btn_serial_off" serial-ctrl-on)
      (form-update-visibility "btn_serial_on" (not serial-ctrl-on))
      (form-update-visibility "btn_usb_off" usb-ctrl-on)
      (form-update-visibility "btn_usb_on" (not usb-ctrl-on)))

    ; Update widgets status
    (widgets-update)))


; Change control
(define (control subsys enabled)

  (format #t "[ajax,portsctrl] change control for ~S to ~S\n" subsys enabled)

  (catch/message
    (lambda()
      (woo-write "/portsctrl/ctrl"
                 'subsys subsys
                 'enabled enabled
                 'language (form-value "language"))))

  (read-current-status))


; Turn ON serial control
(define (serial-ctrl-on)
  (control "serial" #t))


; Turn OFF serial control
(define (serial-ctrl-off)
  (control "serial" #f))


; Turn ON USB control
(define (usb-ctrl-on)
  (control "usb" #t))


; Turn OFF USB control
(define (usb-ctrl-off)
  (control "usb" #f))


; Save serial port settings
(define (serial-save)

  (format #t "[ajax,portsctrl] Save serial port settings\n")

  (catch/message
    (lambda()
      (woo-write "/portsctrl/serial"
                 'port (form-value "list_serial")
                 'enabled (form-value "list_serial_enabled")
                 'owner (form-value "input_serial_owner")
                 'group (form-value "input_serial_group")
                 'mode (form-value "list_serial_mode")
                 'language (form-value "language"))))

  (read-current-status))


; Remove selected entry from USB whitelist
(define (usb-rule-remove)

  (format #t "[ajax,portsctrl] Remove selected rule from whitelist\n")

  (catch/message
    (lambda()
      (woo-write "/portsctrl/usb_rm"
                 'rule_num (f-get-ci-value "list_usb_rules")
                 'language (form-value "language"))))

  (read-current-status))



; Add new USB rule to white list
(define (usb-rule-save)

  (format #t "[ajax,portsctrl] Save USB device settings.\n")

  (let ((num (f-get-ci-value "list_usb_rules")))
    (if (string? num)
	  (catch/message
	   (lambda()
		 (woo-write "/portsctrl/usb_save"
					'num num
					'vendor_id (form-value "input_usb_vendor_id")
					'product_id (form-value "input_usb_product_id")
					'serial (form-value "input_usb_serial")
					'class (form-value "list_usb_class")
					'subclass (form-value "list_usb_subclass")
					'protocol (form-value "list_usb_protocol")
					'owner (form-value "input_usb_owner")
					'group (form-value "input_usb_group")
					'mode (form-value "list_usb_mode")
					'language (form-value "language"))))))

  (read-current-status))

(define (usb-rule-reset)
  (for-each
   (lambda(p) (form-update-value p ""))
   '("list_usb_rules"
     "input_usb_vendor_id" "input_usb_product_id"
	 "list_usb_class" "list_usb_subclass" "list_usb_protocol"
     "input_usb_serial" "input_usb_owner" "input_usb_group"
     "list_usb_mode")))

; Add present USB device to white list
(define (prsnt-dev-add)
  (format #t "[ajax,portsctrl] Add present USB device to whitelist.\n")

  (let* ((dev_path (f-get-ci-value "list_prsnt_devices"))
		 (dev_plist (find-prsnt-device dev_path))
		 (vendor_id (and dev_plist (plistq 'label_prsnt_usb_idvendor
										   (cdr dev_plist))))
		 (product_id (and dev_plist (plistq 'label_prsnt_usb_idproduct
											(cdr dev_plist))))
		 (serial (and dev_plist (plistq 'label_prsnt_usb_serial
											(cdr dev_plist))))
		 (iface (and dev_plist (plistq 'label_prsnt_usb_iface
											(cdr dev_plist))))
		 (class (and dev_plist (plistq 'label_prsnt_usb_class
											(cdr dev_plist))))
		 (subclass (and dev_plist (plistq 'label_prsnt_usb_subclass
											(cdr dev_plist))))
		 (protocol (and dev_plist (plistq 'label_prsnt_usb_proto
											(cdr dev_plist))))
		 (info (and dev_plist
					(find (lambda (s)
							(and s (not (equal? "" (cdr s)))))
						  (if (and iface (not (equal? "" (cdr iface))))
							(list
							 (and protocol
								  (not (equal? "00"
											   (cdr protocol)))
								  (plistq 'label_prsnt_usb_proto_name
										  (cdr dev_plist)))
							 (and subclass
								  (not (equal? "00"
											   (cdr subclass)))
								  (plistq 'label_prsnt_usb_subclass_name
										  (cdr dev_plist)))
							 (and class
								  (not (equal? "00"
											   (cdr class)))
								  (plistq 'label_prsnt_usb_class_name
										  (cdr dev_plist))))
							(list
							 (plistq 'label_prsnt_usb_product
									 (cdr dev_plist))
							 (plistq 'label_prsnt_usb_vendor
									 (cdr dev_plist))))))))

	(if (or (not dev_plist) (null? dev_plist))
      (form-error "No device found for ~A" dev_path)
	  (begin
	    (catch/message
	     (lambda()
	       (apply woo-write "/portsctrl/usb_save"
	    		  'language (form-value "language")
	    		  'info (and info (cdr info))
	    		  (if (and iface (not (equal? "" (cdr iface))))
	    			(list
	    			 'class (and class (cdr class))
	    			 'subclass (and subclass (cdr subclass))
	    			 'protocol (and protocol (cdr protocol)))
	    			(list
	    			 'vendor_id (and vendor_id (cdr vendor_id))
	    			 'product_id (and product_id (cdr product_id))
	    			 'serial (and serial (cdr serial))))))))))

  (read-current-status))


; Edit serial port setting
(define (serial-edit)
  (format #t "[ajax,portsctrl] Edit serial port settings\n")

  (let* ((port (f-get-ci-value "list_serial")))
    (and
      (string? port)

      ; Read current port settings
      (let* ((settings
               (woo-read-first "/portsctrl/serial_port"
                               'port port
                               'language (form-value "language"))))
        (form-update-value-list '("label_serial_selected"
                                  "list_serial_enabled"
                                  "input_serial_owner"
                                  "input_serial_group"
                                  "list_serial_mode")
                                settings))

      (enable-widgets *serial-edit-inputs* *serial-edit-buttons*))))

(define (update-usb-classes)
  (catch/message
   (lambda()
	 (form-update-enum "list_usb_class"
					   (woo-list "/portsctrl/list_usb_classes"
								 'language (form-value "language"))))))

(define (update-usb-subclasses class)
  (catch/message
   (lambda()
	 (form-update-enum "list_usb_subclass"
					   (woo-list "/portsctrl/list_usb_subclasses"
								 'class class
								 'language (form-value "language"))))))

(define (update-usb-protocols class subclass)
  (catch/message
   (lambda()
	 (form-update-enum "list_usb_protocol"
					   (woo-list "/portsctrl/list_usb_protocols"
								 'class class
								 'subclass subclass
								 'language (form-value "language"))))))

(define (usb-edit)
  (format #t "[ajax,portsctrl] Edit USB device settings.\n")

  (let* ((num (f-get-ci-value "list_usb_rules")))
    (and
      (string? num)

      ; Read current device settings
      (let* ((rule
               (woo-read-first "/portsctrl/usb_device"
                               'num num
                               'language (form-value "language"))))
		(update-usb-classes)
		(update-usb-subclasses
		 (let ((a_class (plistq 'list_usb_class (cdr rule))))
		   (and a_class
				(cdr a_class))))
		(update-usb-protocols
		 (let ((a_class (plistq 'list_usb_class (cdr rule))))
		   (and a_class
				(cdr a_class)))
		 (let ((a_subclass (plistq 'list_usb_subclass (cdr rule))))
		   (and a_subclass
				(cdr a_subclass))))
        (form-update-value-list '("input_usb_vendor_id"
                                  "input_usb_product_id"
                                  "input_usb_serial"
								  "list_usb_class"
								  "list_usb_subclass"
								  "list_usb_protocol"
                                  "input_usb_owner"
                                  "input_usb_group"
                                  "list_usb_mode")
                                rule))))

  (enable-widgets *usb-edit-inputs* *usb-edit-buttons*))


; Initialization routine
(define (init)

  (format #t "[ajax,portsctrl] Run initialization routine\n")

  ; Interconnect button with action
  (form-bind "btn_serial_save" "click" serial-save)
  (form-bind "btn_usb_rule_remove" "click" usb-rule-remove)
  (form-bind "btn_usb_save_rule" "click" usb-rule-save)
  (form-bind "btn_usb_reset_rule" "click" usb-rule-reset)
  (form-bind "btn_prsnt_dev_add" "click" prsnt-dev-add)
  (form-bind "btn_prsnt_scan" "click" (lambda() (prsnt-scan #t)))
  (form-bind "bool_allow_hid" "change" allow-usb-hid)
  (form-bind "list_ifaces" "change" (lambda()
									  (js 'ports_access_show_hide_ifaces)
									  (disable-widgets *prsnt-buttons*)))

  (form-bind "btn_serial_on" "click" serial-ctrl-on)
  (form-bind "btn_serial_off" "click" serial-ctrl-off)
  (form-bind "btn_usb_on" "click" usb-ctrl-on)
  (form-bind "btn_usb_off" "click" usb-ctrl-off)

  (form-bind "list_usb_class" "change"
			 (lambda ()
			   (update-usb-subclasses (form-value "list_usb_class"))
			   (form-update-value "list_usb_subclass" "00")))

  (form-bind "list_usb_subclass" "change"
			 (lambda ()
			   (update-usb-protocols
				(form-value "list_usb_class")
				(form-value "list_usb_subclass"))
			   (form-update-value "list_usb_protocol" "00")))

  ; Interface initialization
  (read-current-status))

; vim: ft=lisp expandtab
